package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Representing a computer game's character positions.</p>
 *
 * <p>The game's grid:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>A 11×11 grid</caption>
 *     <colgroup>
 *         <col style="width: 50%;">
 *         <col style="width: 50%;">
 *     </colgroup>
 *     <tr>
 *         <td>
 *             <p>The precise nature of our characters (e.g. Knights, Kings,...) is not
 *important yet. But they'll »live« on our grid's intersection points.</p>
 *          <p>The diagram below shows two positions in
 *            <code style="color: red;">(x|y)</code> notation.</p>
 *          </td>
 *
 *         <td>
 *             <p>A character can move into {@linkplain Direction eight compass directions} starting from its
 *»current« <code>(x|y)</code> position.</p></td>
 *     </tr>
 *
 *     <tr>
 *         <td>
 *            <img src="doc-files/grid.svg" alt="Grid">
 *         </td>
 *         <td>
 *           <img src="doc-files/move.svg" alt="Moving">
 *         </td>
 *     </tr>
 * </table>
 *
 * <p>Characters already residing on the grid's border cannot leave the grid's confines. In the
 * above left diagram for example the character already sitting at <code style="color: red">(6|10)</code> cannot move
 * in northern, northeastern or nortwestern direction anymore. Any such attempt will be flagged as unsuccessful.</p>
 *
 * <p>The following example illustrates the expected behaviour:</p>
 *
 * <table class="goikTableDefaults">
 *  <caption>Moving character positions</caption>
 *  <tr>
 *    <th>Code</th>
 *    <th>Result</th>
 *  </tr>
 *  <tr>
 *    <td>
 *      <pre><code class="java"> final Position jim =
 *     new Position(3,2);               // Starting at (3|2)
 *
 * System.out.println("Start: " + jim); // Output »Start: (3|2)«
 *
 * jim.move(Direction.S);               // Moving in {@link Direction#S} to (3|1)
 * jim.move(Direction.SW);              // Moving in {@link Direction#SW} to (2|0)
 *
 * System.out.println(jim);             // Output: »(2|0)«
 *
 * System.out.println(                  // Already on southmost position:
 *     "Result moving SE: " +           // Attempt leaving grid in
 * jim.move(Direction.SE));             // {@link Direction#SE} fails.
 *
 * System.out.println(jim);             // Unchanged position (2|0)</code></pre>
 *    </td>
 *    <td>
 *      <pre><code class="nohighlight"> Start: (3|2)
 * (2|0)
 * Result moving SE: false
 * (2|0)</code></pre>
 *    </td>
 *
 *   </tr>
 * </table>
 */
public class Position {

    static final int GRIDMAX = 10;// Maximum index value both in x- and y-direction.

    private int x, y;

    /**
     * <p>Creating a new position by virtue of (x|y).</p>
     * @param x The position's x-coordinate value.
     * @param y The position's y-coordinate value.
     */
    public Position(final int x, final int y) {
        this.x = x;
        this.y = y;
    }

    /**
     * <p>Moving one step from current position in the desired direction.</p>
     *
     * <p>Precondition: Leaving the grid is impossible. A position e.g. on the grid's »bottom« cannot be moved
     * southbound. If a move is thus impossible the current position remains unchanged.</p>
     *
     * @param d The desired direction of movement.
     *
     * @return <code>true</code> if the intended move is possible, <code>false</code> otherwise.
     */

    public boolean move(final Direction d) {
        switch(d) {
            case N:
                if (y < GRIDMAX) {
                    y++;
                    return true;
                }
                break;
            case W:
                if (0 < x) {
                    x--;
                    return true;
                }
                break;
            case S:
                if (0 < y) {
                    y--;
                    return true;
                }
                break;
            case E:
                if (x < GRIDMAX) {
                    x++;
                    return true;
                }
                break;
            case NW:
                if (y < GRIDMAX && 0 < x) {
                    y++;
                    x--;
                    return true;
                }
                break;
            case SW:
                if (0 < y && 0 < x) {
                    y--;
                    x--;
                    return true;
                }
                break;
            case SE:
                if (0 < y && x < GRIDMAX) {
                    y--;
                    x++;
                    return true;
                }
                break;
            case NE:
                if (y < GRIDMAX && x < GRIDMAX) {
                    y++;
                    x++;
                    return true;
                }
                break;
        }
        return false;
    }

    @Override
    public String toString() {
        return "("+ x + "|"+ y + ")";
    }
}
